/***************************************************
* By CCSC compiler
*  File Name: 20160820_K457_MK329_1827.c
*  Description:
 * rev 1.0
 * COPYRIGHT 2016 MYCOMKITS.COM, owned by CNET LIMITED
***************************************************/
//
#include <16f1827.h>
#DEVICE ADC=10
// settings
#fuses	HS, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses	NOCPD, NOCLKOUT, NOWRT, NOIESO, NOFCMEN, PLL_SW, NOSTVREN
#use	delay(CLOCK = 4000000)
#USE PWM(OUTPUT=PIN_B6, FREQUENCY=4KHz)
#USE PWM(OUTPUT=PIN_B3)
#USE PWM(OUTPUT=PIN_A3)
#INCLUDE <math.h>
//#USE	RS232(BAUD = 9600, XMIT = PIN_B7)
#byte WPUA = 0x20C
#byte WPUB = 0x20D
#byte OPTION_REG = 0x095
#byte APFCON0 = 0x11D
#byte APFCON1 = 0x11E
#byte T1GCON = 0x19
//
#define	Bmode 0x81			//port B all output, except B0, 7
#define Amode 0xE7			//port A all input except A3,4
//Port define
#define RELAY PIN_B4
#define S1 PIN_B2		//Strobe or Enable Signal
#define S0 PIN_B1
#define SENSOR PIN_B0
#define R_H_SW PIN_A5
#define S2 PIN_A4
#define S3 PIN_B5
#define GET_SW PIN_B7
// valiables
long value_red;
long value_green;
long value_blue;
long get_red;
long get_blue;
long get_green;
long set_red;
long set_blue;
long set_green;
float coeff_red = 0.3;
float coeff_green = 0.3;
float coeff_blue = 0.3;
int coeff_detect = 100;
long hyst;
//
#separate
void initialize();
#separate
void get_settings();
#separate
void covert_hsv();
#separate
void get_color();
#separate
void check_color();
//#separate
//void get_color2();
#separate
void get_one_color();
// main looop
void main()
{
    initialize();
    get_color(); 
    get_settings();
    //
    while(1)
    {
        if(input(GET_SW) == 1)
        {
            get_color();    // get color from TCS230 sensor
            set_red = get_red;
            set_green = get_green;
            set_blue = get_blue;
        }
        else
        {
            get_color();    // get color from TCS230 sensor
            get_settings(); // get settings from 3 pots
            set_red = value_red;
            set_green = value_green;
            set_blue = value_blue;
        }
        set_red = (float)set_red  * coeff_red;
        set_green = (float)set_green  * coeff_green;
        set_blue = (float)set_blue  * coeff_blue;
        set_pwm1_duty(set_red);   //red
        set_pwm2_duty(set_blue);   //blue
        set_pwm3_duty(set_green);   //green
        //
        check_color();
    }
}
//
#separate
void initialize()
{
    set_tris_a(Amode);	//all input
    set_tris_b(Bmode);	//all output
    //
    #use fast_io(A)
    #use fast_io(B)
    // A/D converter initialize
    setup_adc(ADC_CLOCK_DIV_8);	// 2usec
    setup_adc_ports(sAN0 | sAN1 |sAN2 | VSS_VDD);    //
    //
    OPTION_REG = OPTION_REG & 0x7F;	// set 0 to WPUEN
    WPUA = 0x20;	// pull up RA5
    WPUB = 0x80;	// pull up RB7
    //	PWM control initializing 
    setup_ccp1(CCP_PWM | CCP1_B3);  //red
    setup_ccp2(CCP_PWM | CCP2_B6);  //blue
    setup_ccp3(CCP_PWM);    //green
    APFCON0 = APFCON0 & 0xF9;	//
    //
    output_low(RELAY);
    setup_timer_1(T1_FOSC|T1_DIV_BY_1|T1_GATE_SINGLE); //0.25us@4MHz
    // 2% scaling, pulse width 250us to 2500us
    output_low(S0);
    output_high(S1);
    //
    set_pwm1_duty(0);
    set_pwm2_duty(0);
    set_pwm3_duty(0);
    //set_timer1(0xEC78);              // timer 1 setting almost 10msec
    //enable_interrupts(INT_TIMER1);   // timer 1 enable
    //enable_interrupts(GLOBAL);       // global enable
}
//
#separate
void covert_hsv()
{
    float h_floor, fv, pv, qv, tv, bright, saturate;
    int hf_i;
    bright = value_blue; // 0 to 1023
    saturate = (float)value_green / 1023;   // 0 to 1
    h_floor = (float)value_red/169; // =1023/6
    hf_i = (int)floor(h_floor);
    fv = h_floor - hf_i;    // 0 to 1.0
    pv = bright * (1 - saturate); // 0 to 1023
    qv = bright * (1 - fv * saturate); // 0 to 1023
    tv = bright * (1- (1-fv) * saturate); // 0 to 1023
    switch(hf_i)
    {
        case 0:
            value_red = bright;
            value_green = tv;
            value_blue = pv;
            break;
        case 1:
            value_red = qv;
            value_green = bright;
            value_blue = pv;
            break;
        case 2:
            value_red = pv;
            value_green = bright;
            value_blue = tv;
            break;
        case 3:
            value_red = pv;
            value_green = qv;
            value_blue = bright;
            break;
        case 4:
            value_red = tv;
            value_green = pv;
            value_blue = bright;
            break;
        case 5:
            value_red = bright;
            value_green = pv;
            value_blue = qv;
            break;
        case 6:
            value_red = bright;
            value_green = tv;
            value_blue = pv;
            break;
    }
}
//set color to detect
#separate
void get_settings()
{
	set_adc_channel(2);	//must wait 65u
	delay_us(100);
	value_red = read_adc();	//0 to 1023
    set_adc_channel(0);	//must wait 65u
	delay_us(100);
	value_blue = read_adc();	//0 to 1023
    set_adc_channel(1);	//must wait 65u
	delay_us(100);
	value_green = read_adc();	//0 to 1023
    if(input(R_H_SW) == 1)  // 1 = HSV
    {
        covert_hsv();
    }
}
//
#separate
void get_color()
{
    //red
    output_low(S2);
    output_low(S3);
    get_one_color();
    get_red = get_timer1()/10; // 10 to 1000
    if(get_red > 1023) get_red = 1023;
    get_red = 1023 - get_red;   // 23 to 1010
    //blue
    output_low(S2);
    output_high(S3);
    get_one_color();
    get_blue = get_timer1()/10; // 10 to 1000
    if(get_blue > 1023) get_blue = 1023;
    get_blue = 1023 - get_blue;   // 23 to 1010
    //green
    output_high(S2);
    output_high(S3);
    get_one_color();
    get_green = get_timer1()/10; // 10 to 1000
    if(get_green > 1023) get_green = 1023;
    get_green = 1023 - get_green;   // 23 to 1010
}
//
#separate
void get_one_color()
{
    T1GCON = T1GCON | 0x08;
    while((T1GCON & 0x04) == 0) {}  // hold during zero
    set_timer1(0); 
    while((T1GCON & 0x04) != 0) {}  // count up during not zero
}
//
#separate
void check_color()
{
    int same_color = 0;
    long tempo = 10000;
    long tempo_v_red;
    long tempo_v_green;
    long tempo_v_blue;
    long tempo_g_red;
    long tempo_g_green;
    long tempo_g_blue;
    //
    if(tempo >= value_red) tempo = value_red;
    if(tempo >= value_green) tempo = value_green;
    if(tempo >= value_blue) tempo = value_blue;
    tempo_v_red = value_red - tempo;
    tempo_v_green = value_green - tempo;
    tempo_v_blue = value_blue - tempo;
    //
    tempo = 10000;
    if(tempo >= get_red) tempo = get_red;
    if(tempo >= get_green) tempo = get_green;
    if(tempo >= get_blue) tempo = get_blue;
    tempo_g_red = get_red - tempo;
    tempo_g_green = get_green - tempo;
    tempo_g_blue = get_blue - tempo;
    //
    if(tempo_g_red <= (tempo_v_red + coeff_detect))
    {
        if(tempo_v_red < coeff_detect) tempo = 0;
        else tempo = tempo_v_red - coeff_detect;
        if(tempo_g_red >= tempo) same_color = same_color + 1;
    }
    //
    if(tempo_g_green <= (tempo_v_green + coeff_detect))
    {
        if(tempo_v_green < coeff_detect) tempo = 0;
        else tempo = tempo_v_green - coeff_detect;
        if(tempo_g_green >= tempo) same_color = same_color + 1;
    }
    //
    if(tempo_g_blue <= (tempo_v_blue + coeff_detect))
    {
        if(tempo_v_blue < coeff_detect) tempo = 0;
        else tempo = tempo_v_blue - coeff_detect;
        if(tempo_g_blue >= tempo) same_color = same_color + 1;
    }
    //
    if(same_color == 3) 
    {
        output_high(RELAY);
        hyst = 100;
    }
    else 
    {
        if(hyst == 0) output_low(RELAY);
    }
    if(hyst > 0) hyst = hyst - 1;
    else hyst = 0;
}
